<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class MinhasTarefasFactory
{

    use TQuery;
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection       = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }

    // Funções da tela de listagem ed tarefas
    public function getTarefasListadas($listaOperadores, $filtros)
    {

        $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND, CA.NRSEQCONTRATO, LC.CDCLIENTE,
                    TO_CHAR(CA.DTHORAAGEND, 'DD/MM') DTAGEND,  TO_CHAR(CA.DTHORAAGEND, 'HH24:MI') HORAAGEND, CA.NMCHECKLISTAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
                    LCP.NMLOCALCLIENTE AS NMLOCALPAI, CHK.NMCHECKLIST, F.NMOPERADOR, C.NRCONTRATOFCT, CT.NMRAZSOCCLIE,
                                    CASE CA.IDSITUACAOAGEND 
                                    WHEN '1' THEN 'Pendente'
                                    WHEN '2' THEN 'Em andamento'
                                    WHEN '3' THEN 'Cancelado'
                                    WHEN '4' THEN 'Concluído'
                                    WHEN '5' THEN 'Atrasado'
                                    ELSE NULL END AS SITUACAO
                                FROM FCM_CHECKLISTAGEND CA
                                JOIN FCM_LOCALCLI LC
                                ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                                JOIN FCM_LOCALCLI LCP
                                ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                                JOIN CLIENTE CT
                                ON CT.CDCLIENTE = LC.CDCLIENTE
                                JOIN FCM_CHECKLIST CHK
                                ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                                JOIN OPERADOR F
                                ON F.CDOPERADOR = CA.CDOPERADOR
                                LEFT JOIN FCT_CONTRATO C
                                ON C.NRSEQCONTRATO = CA.NRSEQCONTRATO
                                WHERE CA.CDOPERADOR IN ({$listaOperadores})" . $filtros . "
                             ORDER BY CA.DTHORAAGEND";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }


    public function getFotosTarefas($parametros)
    {

        $sql = "SELECT FCM_FOTOTAREFAAGEND.NRFOTOTAREFAAGEND, FCM_FOTOTAREFAAGEND.DSFOTOTAREFAAGEND, FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND
                    FROM FCM_FOTOTAREFAAGEND
                    JOIN FCM_TAREFAAGEND 
                    ON FCM_TAREFAAGEND.NRTAREFAAGEND = FCM_FOTOTAREFAAGEND.NRTAREFAAGEND
                    JOIN FCM_CHECKLISTAGEND 
                    ON FCM_CHECKLISTAGEND.NRCHECKLISTAGEND = FCM_TAREFAAGEND.NRCHECKLISTAGEND
                    WHERE FCM_CHECKLISTAGEND.NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function excluirFotoTarefas($parametros)
    {

        $sql = "DELETE FROM FCM_FOTOTAREFAAGEND
                    WHERE NRFOTOTAREFAAGEND = :NRFOTOTAREFAAGEND";

        $this->connection->executeStatement($sql, $parametros);
    }

    //Funções da Tela Minhas Tarefas
    public function getCodTarefaAgend($parametros)
    {

        $sql = "SELECT NRTAREFAAGEND
                    FROM FCM_TAREFAAGEND
                    WHERE NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function anexarImagemTarefas($parametros)
    {

        $sql = "INSERT INTO FCM_FOTOTAREFAAGEND (NRFOTOTAREFAAGEND, NRTAREFAAGEND, DSFOTOTAREFAAGEND, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO, DSTYPE, NRSIZE)
        VALUES (:NRFOTOTAREFAAGEND, :NRTAREFAAGEND, :DSFOTOTAREFAAGEND, 'S', :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO, :DSTYPE, :NRSIZE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getMinhasTarefas($listaOperadores, $filtros)
    {

        $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND, CA.NRSEQCONTRATO, LC.CDCLIENTE,
        TO_CHAR(CA.DTHORAAGEND, 'DD/MM') DTAGEND,  TO_CHAR(CA.DTHORAAGEND, 'HH24:MI') HORAAGEND, CA.NMCHECKLISTAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
        LCP.NMLOCALCLIENTE AS NMLOCALPAI, CHK.NMCHECKLIST, F.NMOPERADOR, C.NRCONTRATOFCT, CT.NMRAZSOCCLIE, CA.IDOBRIGAOBSERVACAO,
                  CASE CA.IDSITUACAOAGEND WHEN '1' THEN 'Pendente'
                  WHEN '2' THEN 'Em andamento'
                  WHEN '5' THEN 'Atrasado'
                  ELSE NULL END AS SITUACAO
              FROM FCM_CHECKLISTAGEND CA
              JOIN FCM_LOCALCLI LC
                ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
              JOIN FCM_LOCALCLI LCP
                ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
              JOIN CLIENTE CT
                ON CT.CDCLIENTE = LC.CDCLIENTE
              JOIN FCM_CHECKLIST CHK
                ON CA.NRCHECKLIST = CHK.NRCHECKLIST
              JOIN OPERADOR F
                ON F.CDOPERADOR = CA.CDOPERADOR
              LEFT JOIN FCT_CONTRATO C
                ON C.NRSEQCONTRATO = CA.NRSEQCONTRATO
              WHERE CA.IDSITUACAOAGEND IN (1,2,5)
              AND CA.IDATIVO = 'S'
              AND CA.CDOPERADOR IN ({$listaOperadores})" . $filtros . "
              ORDER BY CA.IDSITUACAOAGEND DESC, CA.DTHORAAGEND ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTarefasPorOperador($operadores)
    {

        $sql = "SELECT NMCHECKLISTAGEND || ' - ' || TO_CHAR(DTHORAAGEND, 'DD/MM HH24:MI') AS NMCHECKLISTAGEND, NRCHECKLISTAGEND
                    FROM FCM_CHECKLISTAGEND
                    WHERE CDOPERADOR IN ({$operadores})
                    AND IDSITUACAOAGEND IN ('1', '2')
                    AND IDATIVO = 'S'";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function itensDetailsTaskForVistoria($parametros)
    {
        $resposta = array(
            "observation" => [],
            "criterion" => []
        );
        $sql = "SELECT DISTINCT TA.DSOBSTAREFAAGEND
               FROM FCM_TAREFAAGEND TA
               JOIN FCM_CHECKLISTAGEND  CA
                 ON TA.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
              WHERE TA.DSOBSTAREFAAGEND IS NOT NULL
                AND 
                    CA.NRCHECKLISTAGEND = (SELECT NRCHECKLISTAGEND
                                             FROM FCM_GERENCIAMENTOCHK
                                            WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK)";

        $resposta['observation'] =  $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();

        $sql = "SELECT DISTINCT TF.NMTAREFA, TA.IDREALIZADO
               FROM FCM_TAREFAAGEND TA
               JOIN FCM_CHECKLISTAGEND  CA
                 ON TA.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
               JOIN FCM_TAREFACHK T
                 ON T.NRTAREFACHK = TA.NRTAREFACHK
               JOIN FCM_TAREFA TF
                 ON T.NRTAREFA = TF.NRTAREFA
              WHERE CA.NRCHECKLISTAGEND = (SELECT NRCHECKLISTAGEND 
                                             FROM FCM_GERENCIAMENTOCHK
                                            WHERE NRGERENCIAMENTOCHK = :NRGERENCIAMENTOCHK)";

        $resposta['criterion'] =  $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
        return $resposta;
    }


    // Update para alterar o status da tarefa
    // Quando iniciar a tarefa , passar o valor '2' / Quando finalizar, passar o valor '4'  / Quando reportar, passar o valor 3
    // DSJUSTCANCELAMENTO = Deve ser preenchido somente quando for reportar. Para as outras ações, deve ser passado "NULL"
    public function alterarStatusTarefa($parametros)
    {

        $sql = "UPDATE FCM_CHECKLISTAGEND
                    SET IDSITUACAOAGEND = :IDSITUACAOAGEND,
                    DTULTATU = SYSDATE,
                    DSJUSTCANCELAMENTO = :DSJUSTCANCELAMENTO, 
                    CDOPERULTATU = :CDOPERULTATU, 
                    NRORGULTATU = :NRORGULTATU
                    WHERE NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function upStatusVistoryCanc($parametros)
    {

        $sql = "UPDATE FCM_GERENCIAMENTOCHK
                SET IDSITUACAOGERCHK = :IDSITUACAOGERCHK,
                    DTULTATU = SYSDATE,
                    CDOPERULTATU = :CDOPERULTATU, 
                    NRORGULTATU = :NRORGULTATU
                WHERE NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        $this->connection->executeStatement($sql, $parametros);
    }


    // Update de finalização do detalhamento
    // IDREALIZADO = Somente dos checks que foram marcados
    public function alterarStatusItens($parametros)
    {

        $sql = "UPDATE FCM_TAREFAAGEND
                    SET IDREALIZADO = 'S',
                        DTULTATU = SYSDATE,
                        DSOBSTAREFAAGEND = :DSOBSTAREFAAGEND
                WHERE NRTAREFAAGEND = :NRTAREFAAGEND
                AND NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        $this->connection->executeStatement($sql, $parametros);
    }

    // Funções da Tela de Detalhamento
    public function getItensTarefas($parametros)
    {

        $sql = "SELECT TA.NRTAREFAAGEND, TA.NRCHECKLISTAGEND, TA.CDOPERADOR, TO_CHAR(TA.DTHORATAREFAAGEND, 'DD/MM HH24:MI') DTHORATAREFAAGEND,
              TA.NRTAREFACHK, TA.IDREALIZADO, TF.NMTAREFA
               FROM FCM_TAREFAAGEND TA
               JOIN FCM_CHECKLISTAGEND  CA
               ON TA.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
               JOIN FCM_TAREFACHK T
               ON T.NRTAREFACHK = TA.NRTAREFACHK
               JOIN FCM_TAREFA TF
               ON T.NRTAREFA = TF.NRTAREFA
               WHERE CA.NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getDetalhesTarefa($parametros)
    {

        $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND,
        TO_CHAR(CA.DTHORAAGEND, 'DD/MM HH24:MI') DTHORAAGEND, CA.NMCHECKLISTAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
        LCP.NMLOCALCLIENTE AS NMLOCALPAI, CHK.NMCHECKLIST, F.NMOPERADOR, CA.IDOBRIGAOBSERVACAO,
                  CASE CA.IDSITUACAOAGEND WHEN '1' THEN 'Pendente'
                  WHEN '2' THEN 'Em andamento'
                  WHEN '5' THEN 'Atrasado'
                  ELSE NULL END AS IDSITUACAOAGEND
              FROM FCM_CHECKLISTAGEND CA
              JOIN FCM_LOCALCLI LC
              ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
              JOIN FCM_LOCALCLI LCP
              ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
              JOIN FCM_CHECKLIST CHK
              ON CA.NRCHECKLIST = CHK.NRCHECKLIST
              JOIN OPERADOR F
              ON F.CDOPERADOR = CA.CDOPERADOR
              WHERE CA.NRCHECKLISTAGEND = :NRCHECKLISTAGEND";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    public function qtdTarefasAtivas($listaOperadores)
    {
        $sql = "SELECT COUNT (IDSITUACAOAGEND) AS TAREFAATIVA
                FROM FCM_CHECKLISTAGEND
                WHERE IDSITUACAOAGEND = '2'
                AND CDOPERADOR IN ({$listaOperadores})";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function statusCheck($parametros)
    {

        $sql = "SELECT COUNT(IDSITUACAOAGEND)
               FROM FCM_CHECKLISTAGEND
               WHERE IDSITUACAOAGEND = '2'
               AND CDOPERADOR = :CDOPERADOR";

        return $this->connection->executeQuery($sql, $parametros)->fetchAllAssociative();
    }

    // ------------------------------ FUNÇÔES GEOLOCALIZAÇÃO ------------------------------
    public function getGeolocalizacao($params)
    {
        $sql = "SELECT NRLOCALCLIENTE, NRGEOLOCALIZACAO
                FROM FCM_LOCALCLI
                WHERE NRLOCALCLIENTE = $params";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getParametrizacao()
    {
        $sql = "SELECT IDUTILIZAGEOLOCAL, IDUTILIZAGEOLOCALINITAR
        FROM FCM_PARAMGERAL";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }
}
