import {
    Date as ZdDate, Modal, ModalService, Select, TextInput, Textarea, Button, Text, Form, LoadingService, SelectMultiple, FileInput,
} from '@zeedhi/common';
import {
    Metadata, Http, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
    IterableComponentRender, IterableCarousel,
} from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import { Rating } from '@zeedhi/zd-rating';
import FileUpload from './FileUpload';

export class SolicitacoesController {
    public visibilidadeTelaPrincipal: boolean = true;

    public visibilidadeCriarSolicitacao: boolean = false;

    public visibilidadeExibirSolicitacao: boolean = false;

    public visibilidadeAvaliarSolicitacao: boolean = false;

    private filtersModal!: Modal;

    public visualizarCriacaoSolicitacao() {
        this.visibilidadeTelaPrincipal = false;
        this.visibilidadeCriarSolicitacao = !this.visibilidadeTelaPrincipal;
    }

    public voltarPaginaInicial({ component }: any) {
        this.visibilidadeTelaPrincipal = true;
        if (component.name === 'cancelar-criacao-solicitacao') {
            this.visibilidadeCriarSolicitacao = !this.visibilidadeTelaPrincipal;
            this.limparCampos();
        } else {
            this.visibilidadeExibirSolicitacao = !this.visibilidadeTelaPrincipal;
        }
    }

    public visualizarSolicitacao({ component }: any) {
        this.visibilidadeTelaPrincipal = false;
        this.visibilidadeExibirSolicitacao = !this.visibilidadeTelaPrincipal;
        this.montarInformacoesSolicitacoes(component.name.split('_')[1]);
        this.carregarFotosSolicitacao(component.name.split('_')[1]);
    }

    public visualizarAvaliacaoSolicitacao() {
        this.visibilidadeAvaliarSolicitacao = true;
        this.visibilidadeExibirSolicitacao = !this.visibilidadeAvaliarSolicitacao;
    }

    public voltarPaginaVisualizacao() {
        this.visibilidadeAvaliarSolicitacao = false;
        this.visibilidadeExibirSolicitacao = !this.visibilidadeAvaliarSolicitacao;
        Metadata.getInstance<Button>('button-yes').outline = true;
        Metadata.getInstance<Button>('button-no').outline = true;
        Metadata.getInstance<Textarea>('descricoes-avaliacao').value = null;
        Metadata.getInstance<Rating>('ratings').value = 0;
    }

    public async createModal() {
        this.filtersModal = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00020_000_solicitacoes/modal/fm_00020_001_filtrar_solicitacoes' });
    }

    public destroyModal() {
        this.filtersModal?.destroy();
    }

    public openModal() {
        this.filtersModal.show();
    }

    public closeModal() {
        this.filtersModal.hide();
    }

    public filtrarCards() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-solicitacoes');

        iterable.datasource.setFilter({
            NRSOLICITACAO: Metadata.getInstance<SelectMultiple>('selectName').value,
            NRLOCALCLIENTE: Metadata.getInstance<SelectMultiple>('selectLocal').value,
            DTHORAAGEND: Metadata.getInstance<ZdDate>('selectDate').value,
            IDSITUACAO: Metadata.getInstance<SelectMultiple>('selectSituation').value,
            TIPO: 2,
        }).finally(() => {
            this.closeModal();
        });
    }

    public limparFiltros() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-cards-solicitacoes');

        Metadata.getInstance<SelectMultiple>('selectName').value = null;
        Metadata.getInstance<SelectMultiple>('selectLocal').value = null;
        Metadata.getInstance<ZdDate>('selectDate').value = null;
        Metadata.getInstance<SelectMultiple>('selectSituation').value = null;

        this.carregarCardsTarefas({ component: iterable });
    }

    public carregarCardsTarefas({ component }: any) {
        component.datasource.setFilter({
            TIPO: 1,
        });
    }

    public async createNewRequest() {
        const form = Metadata.getInstance<Form>('input-form');
        if (Utilities.validateForm(form)) {
            LoadingService.show();
            const imagensConvertidas = await FileUpload.converterImagens('FileInputSolicitacao');
            await Http.post('/criarSolicitacao', {
                NMSOLICITACAO: Metadata.getInstance<TextInput>('input-nmsolicitacao').value,
                NRLOCALCLIENTE: Metadata.getInstance<Select>('select-local').value,
                DSSOLICITACAO: Metadata.getInstance<Textarea>('input-text-area').value,
                IDSITUACAO: 1,
                IMAGENS: imagensConvertidas,
                CDOPERADOR: localStorage.getItem('CDOPERADOR'),
            }).then(() => {
                Utilities.showSuccessMessage();
                this.visibilidadeTelaPrincipal = true;
                this.visibilidadeCriarSolicitacao = false;
                this.limparCampos();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public limparCampos() {
        Metadata.getInstance<TextInput>('input-nmsolicitacao').value = null;
        Metadata.getInstance<Select>('select-local').value = null;
        Metadata.getInstance<Textarea>('input-text-area').value = null;
        Metadata.getInstance<FileInput>('FileInputSolicitacao').value = null;
    }

    // Função para estilizar os status das solicitações
    public stylizeStatus({ component }: IEventParam<Text>) {
        if (component.text === 'Aguardando Avaliação') {
            component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 12px;;'>AGUARDANDO AVALIAÇÃO</span>";
        }
        if (component.text === 'Concluído') {
            component.text = "<span  style='background-color: #dee2e6; color: green; font-size: 12px;'>CONCLUÍDO</span>";
        }
        if (component.text === 'Em andamento') {
            component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 12px;'>EM ANDAMENTO</span>";
        }
        if (component.text === 'Pendente') {
            component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 12px;'>PENDENTE</span>";
        }
    }

    public montarInformacoesSolicitacoes(nrsolicitacao: number) {
        const card = Metadata.getInstance<IterableComponentRender>('iterable-cards-solicitacoes');
        const { data } = card.datasource;
        const dados: any = data.find((row: any) => row.NRSOLICITACAO === Number(nrsolicitacao));

        Metadata.getInstance<TextInput>('cod-nr').value = dados.NRSOLICITACAO;
        Metadata.getInstance<TextInput>('input-title').value = dados.NMSOLICITACAO;
        Metadata.getInstance<TextInput>('input-local').value = dados.NMLOCALCLIENTE;
        Metadata.getInstance<TextInput>('input-date').value = `${dados.DTAGEND} - ${dados.HORAAGEND}`;
        Metadata.getInstance<TextInput>('input-situation').value = dados.IDSITUACAO;
        Metadata.getInstance<Textarea>('exibir-descricao').value = dados.DSSOLICITACAO;
        Metadata.getInstance<Button>('show-button-evaluate').disabled = (dados.IDSITUACAO !== 'Aguardando Avaliação');
    }

    public changeColor({ component }: { component: Button }) {
        component.outline = false;
        Metadata.getInstance<Button>((component.name === 'button-no') ? 'button-yes' : 'button-no').outline = true;
    }

    public evaluateRequest() {
        const bYes = Metadata.getInstance<Button>('button-yes');
        const bNo = Metadata.getInstance<Button>('button-no');
        const avaliacao = Metadata.getInstance<Rating>('ratings').value;
        const nr = Metadata.getInstance<TextInput>('cod-nr').value;

        if ((!bYes.outline || !bNo.outline) && avaliacao) {
            LoadingService.show();
            Http.post('/avaliarSolicitacao', {
                NRSOLICITACAO: nr,
                IDSATISFACAOSOLICITA: avaliacao,
                DSSOLICITACAO: Metadata.getInstance<Textarea>('descricoes-avaliacao').value,
                CONDITION: !bYes.outline,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
                this.visibilidadeAvaliarSolicitacao = false;
                this.visibilidadeTelaPrincipal = true;
            });
        } else {
            Utilities.alertMessage('Responda se a solicitação foi resolvida!', 'error');
        }
    }

    public carregarFotosSolicitacao(nrsolicitacao: any) {
        const component = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-solicitacoes');

        LoadingService.show();
        Http.post('/getFotosSolicitacao', {
            NRSOLICITACAO: nrsolicitacao,
        }).then((response: any) => {
            if (response.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-tarefas').isVisible = true;
            } else {
                response.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRIMGSOLICITACAO: dados.NRIMGSOLICITACAO,
                        DSCAMINHO: dados.DSCAMINHO,
                    });
                });
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
        });
    }
}
